package dev.fyloz.trial.colorrecipesexplorer.service

import dev.fyloz.trial.colorrecipesexplorer.exception.model.EntityAlreadyExistsRestException
import dev.fyloz.trial.colorrecipesexplorer.exception.model.EntityNotFoundRestException
import dev.fyloz.trial.colorrecipesexplorer.model.Material
import dev.fyloz.trial.colorrecipesexplorer.model.MixType
import dev.fyloz.trial.colorrecipesexplorer.repository.MixTypeRepository
import org.springframework.stereotype.Service

interface MixTypeService : NamedModelService<MixType, MixTypeRepository> {
    /** Gets the mix type with the given [material]. */
    fun getByMaterial(material: Material): MixType
}

@Service
class MixTypeServiceImpl(mixTypeRepository: MixTypeRepository, private val materialService: MaterialService) :
    AbstractNamedModelService<MixType, MixTypeRepository>(mixTypeRepository), MixTypeService {
    override fun getByMaterial(material: Material): MixType =
        repository.findByMaterial(material) ?: throw EntityNotFoundRestException(material.name)

    override fun save(entity: MixType): MixType {
        if (materialService.existsByName(entity.name))
            throw EntityAlreadyExistsRestException(entity.name)
        return super.save(entity)
    }
}
