package dev.fyloz.trial.colorrecipesexplorer.service

import dev.fyloz.trial.colorrecipesexplorer.model.*
import dev.fyloz.trial.colorrecipesexplorer.model.validation.or
import dev.fyloz.trial.colorrecipesexplorer.repository.RecipeRepository
import org.springframework.stereotype.Service
import kotlin.contracts.ExperimentalContracts

interface RecipeService : ExternalModelService<Recipe, RecipeSaveDto, RecipeUpdateDto, RecipeRepository> {
    /** Checks if one or more recipes have the given [company]. */
    fun existsByCompany(company: Company): Boolean

    /** Gets all recipes with the given [company]. */
    fun getAllByCompany(company: Company): Collection<Recipe>
}

@Service
class RecipeServiceImpl(recipeRepository: RecipeRepository) :
    AbstractExternalModelService<Recipe, RecipeSaveDto, RecipeUpdateDto, RecipeRepository>(recipeRepository),
    RecipeService {
    override fun existsByCompany(company: Company): Boolean = repository.existsByCompany(company)
    override fun getAllByCompany(company: Company): Collection<Recipe> = repository.findAllByCompany(company)

    @ExperimentalContracts
    override fun update(entity: RecipeUpdateDto): Recipe {
        val persistedRecipe by lazy { getById(entity.id) }

        return update(with(entity) {
            recipe(
                id = id,
                name = name.or(persistedRecipe.name),
                description = description.or(persistedRecipe.description),
                sample = if (sample != null && sample >= 0) sample else persistedRecipe.sample,
                approbationDate = approbationDate ?: persistedRecipe.approbationDate,
                remark = remark.or(persistedRecipe.remark),
                note = note.or(persistedRecipe.note),
                company = company ?: persistedRecipe.company,
                steps = if (!steps.isNullOrEmpty()) steps else persistedRecipe.steps
            )
        })
    }
}
